@@ -273,14 +273,9 @@ setup_test_address (void)
 {
   if (is_unix)
     {
-      g_test_message ("Testing with unix:tmpdir address");
-      if (g_unix_socket_address_abstract_names_supported ())
-	tmp_address = g_strdup ("unix:tmpdir=/tmp/gdbus-test-");
-      else
-	{
-	  tmpdir = g_dir_make_tmp ("gdbus-test-XXXXXX", NULL);
-	  tmp_address = g_strdup_printf ("unix:tmpdir=%s", tmpdir);
-	}
+      g_test_message ("Testing with unix:dir address");
+      tmpdir = g_dir_make_tmp ("gdbus-test-XXXXXX", NULL);
+      tmp_address = g_strdup_printf ("unix:dir=%s", tmpdir);
     }
   else
     tmp_address = g_strdup ("nonce-tcp:");
@@ -288,11 +283,11 @@ setup_test_address (void)
 
 #ifdef G_OS_UNIX
 static void
-setup_dir_test_address (void)
+setup_tmpdir_test_address (void)
 {
-  g_test_message ("Testing with unix:dir address");
+  g_test_message ("Testing with unix:tmpdir address");
   tmpdir = g_dir_make_tmp ("gdbus-test-XXXXXX", NULL);
-  tmp_address = g_strdup_printf ("unix:dir=%s", tmpdir);
+  tmp_address = g_strdup_printf ("unix:tmpdir=%s", tmpdir);
 }
 
 static void
@@ -313,7 +308,8 @@ teardown_test_address (void)
       /* Ensuring the rmdir succeeds also ensures any sockets created on the
        * filesystem are also deleted.
        */
-      g_assert_cmpint (g_rmdir (tmpdir), ==, 0);
+      g_assert_cmpstr (g_rmdir (tmpdir) == 0 ? "OK" : g_strerror (errno),
+                       ==, "OK");
       g_clear_pointer (&tmpdir, g_free);
     }
 }
@@ -1033,6 +1029,9 @@ do_test_peer (void)
 static void
 test_peer (void)
 {
+  test_guid = g_dbus_generate_guid ();
+  loop = g_main_loop_new (NULL, FALSE);
+
   /* Run this test multiple times using different address formats to ensure
    * they all work.
    */
@@ -1041,7 +1040,7 @@ test_peer (void)
   teardown_test_address ();
 
 #ifdef G_OS_UNIX
-  setup_dir_test_address ();
+  setup_tmpdir_test_address ();
   do_test_peer ();
   teardown_test_address ();
 
@@ -1049,6 +1048,9 @@ test_peer (void)
   do_test_peer ();
   teardown_test_address ();
 #endif
+
+  g_main_loop_unref (loop);
+  g_free (test_guid);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -1064,6 +1066,9 @@ test_peer_signals (void)
 
   g_test_bug ("https://gitlab.gnome.org/GNOME/glib/issues/1620");
 
+  test_guid = g_dbus_generate_guid ();
+  loop = g_main_loop_new (NULL, FALSE);
+
   setup_test_address ();
   memset (&data, '\0', sizeof (PeerData));
   data.current_connections = g_ptr_array_new_with_free_func (g_object_unref);
@@ -1119,6 +1124,9 @@ test_peer_signals (void)
   g_thread_join (service_thread);
 
   teardown_test_address ();
+
+  g_main_loop_unref (loop);
+  g_free (test_guid);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -1246,6 +1254,7 @@ dmp_thread_func (gpointer user_data)
   data->loop = g_main_loop_new (data->context, FALSE);
   g_main_loop_run (data->loop);
 
+  g_dbus_server_stop (data->server);
   g_main_context_pop_thread_default (data->context);
 
   g_free (guid);
@@ -1260,6 +1269,9 @@ delayed_message_processing (void)
   GThread *service_thread;
   guint n;
 
+  test_guid = g_dbus_generate_guid ();
+  loop = g_main_loop_new (NULL, FALSE);
+
   setup_test_address ();
 
   data = g_new0 (DmpData, 1);
@@ -1307,6 +1319,9 @@ delayed_message_processing (void)
   g_thread_join (service_thread);
   dmp_data_free (data);
   teardown_test_address ();
+
+  g_main_loop_unref (loop);
+  g_free (test_guid);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -1405,6 +1420,9 @@ test_nonce_tcp (void)
   gboolean res;
   const gchar *address;
 
+  test_guid = g_dbus_generate_guid ();
+  loop = g_main_loop_new (NULL, FALSE);
+
   memset (&data, '\0', sizeof (PeerData));
   data.current_connections = g_ptr_array_new_with_free_func (g_object_unref);
 
@@ -1512,6 +1530,9 @@ test_nonce_tcp (void)
   g_thread_join (service_thread);
 
   g_ptr_array_unref (data.current_connections);
+
+  g_main_loop_unref (loop);
+  g_free (test_guid);
 }
 
 static void
@@ -1596,6 +1617,9 @@ test_tcp_anonymous (void)
   GDBusConnection *connection;
   GError *error;
 
+  test_guid = g_dbus_generate_guid ();
+  loop = g_main_loop_new (NULL, FALSE);
+
   seen_connection = FALSE;
   service_thread = g_thread_new ("tcp-anon-service",
                                  tcp_anonymous_service_thread_func,
@@ -1623,6 +1647,9 @@ test_tcp_anonymous (void)
   server = NULL;
 
   g_thread_join (service_thread);
+
+  g_main_loop_unref (loop);
+  g_free (test_guid);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -1767,6 +1794,9 @@ codegen_test_peer (void)
   GVariant            *value;
   const gchar         *s;
 
+  test_guid = g_dbus_generate_guid ();
+  loop = g_main_loop_new (NULL, FALSE);
+
   setup_test_address ();
 
   /* bring up a server - we run the server in a different thread to avoid deadlocks */
@@ -1874,6 +1904,9 @@ codegen_test_peer (void)
   g_thread_join (service_thread);
 
   teardown_test_address ();
+
+  g_main_loop_unref (loop);
+  g_free (test_guid);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -1886,17 +1919,12 @@ main (int   argc,
   gint ret;
   GDBusNodeInfo *introspection_data = NULL;
 
-  g_test_init (&argc, &argv, NULL);
+  g_test_init (&argc, &argv, G_TEST_OPTION_ISOLATE_DIRS, NULL);
 
   introspection_data = g_dbus_node_info_new_for_xml (test_interface_introspection_xml, NULL);
   g_assert (introspection_data != NULL);
   test_interface_introspection_data = introspection_data->interfaces[0];
 
-  test_guid = g_dbus_generate_guid ();
-
-  /* all the tests rely on a shared main loop */
-  loop = g_main_loop_new (NULL, FALSE);
-
   g_test_add_func ("/gdbus/peer-to-peer", test_peer);
   g_test_add_func ("/gdbus/peer-to-peer/signals", test_peer_signals);
   g_test_add_func ("/gdbus/delayed-message-processing", delayed_message_processing);
@@ -1906,10 +1934,8 @@ main (int   argc,
   g_test_add_func ("/gdbus/credentials", test_credentials);
   g_test_add_func ("/gdbus/codegen-peer-to-peer", codegen_test_peer);
 
-  ret = g_test_run();
+  ret = g_test_run ();
 
-  g_main_loop_unref (loop);
-  g_free (test_guid);
   g_dbus_node_info_unref (introspection_data);
 
   return ret;
