@@ -1,8 +1,6 @@
 /*
  * Copyright 2019 Collabora Ltd.
  *
- * SPDX-License-Identifier: LGPL-2.1-or-later
- *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -24,9 +22,6 @@
 #include <glib/gstdio.h>
 #include <gio/gio.h>
 
-/* For G_CREDENTIALS_*_SUPPORTED */
-#include <gio/gcredentialsprivate.h>
-
 #ifdef HAVE_DBUS1
 #include <dbus/dbus.h>
 #endif
@@ -39,9 +34,8 @@ typedef enum
   INTEROP_FLAGS_TCP = (1 << 3),
   INTEROP_FLAGS_LIBDBUS = (1 << 4),
   INTEROP_FLAGS_ABSTRACT = (1 << 5),
-  INTEROP_FLAGS_REQUIRE_SAME_USER = (1 << 6),
   INTEROP_FLAGS_NONE = 0
-} G_GNUC_FLAG_ENUM InteropFlags;
+} InteropFlags;
 
 static gboolean
 allow_external_cb (G_GNUC_UNUSED GDBusAuthObserver *observer,
@@ -155,7 +149,6 @@ whoami_filter_cb (GDBusConnection *connection,
       g_dbus_connection_send_message (connection, reply,
                                       G_DBUS_SEND_MESSAGE_FLAGS_NONE,
                                       NULL, NULL);
-      g_object_unref (reply);
 
       /* handled */
       g_object_unref (message);
@@ -210,7 +203,6 @@ libdbus_call_task_cb (GTask *task,
                                                                    libdbus_call->call,
                                                                    -1,
                                                                    &libdbus_call->error);
-  g_task_return_boolean (task, TRUE);
 }
 #endif /* HAVE_DBUS1 */
 
@@ -248,11 +240,6 @@ assert_expected_uid_pid (InteropFlags flags,
        * on Linux. */
       g_assert_cmpint (uid, ==, getuid ());
       g_assert_cmpint (pid, ==, getpid ());
-#elif defined(__APPLE__)
-      /* We know (or at least suspect) that both GDBus and libdbus support
-       * passing the uid only on macOS. */
-      g_assert_cmpint (uid, ==, getuid ());
-      /* No pid here */
 #else
       g_test_message ("Please open a merge request to add appropriate "
                       "assertions for your platform");
@@ -329,8 +316,6 @@ do_test_server_auth (InteropFlags flags)
 
   if (flags & INTEROP_FLAGS_ANONYMOUS)
     server_flags |= G_DBUS_SERVER_FLAGS_AUTHENTICATION_ALLOW_ANONYMOUS;
-  if (flags & INTEROP_FLAGS_REQUIRE_SAME_USER)
-    server_flags |= G_DBUS_SERVER_FLAGS_AUTHENTICATION_REQUIRE_SAME_USER;
 
   observer = g_dbus_auth_observer_new ();
 
@@ -409,14 +394,6 @@ do_test_server_auth (InteropFlags flags)
       LibdbusCall libdbus_call = { DBUS_ERROR_INIT, NULL, NULL, NULL };
       GTask *task;
 
-      /* The test suite uses %G_TEST_OPTION_ISOLATE_DIRS, which sets
-       * `HOME=/dev/null` and leaves g_get_home_dir() pointing to the per-test
-       * temp home directory. Unfortunately, libdbus doesn’t allow the home dir
-       * to be overridden except using the environment, so copy the per-test
-       * temp home directory back there so that libdbus uses the same
-       * `$HOME/.dbus-keyrings` path as GLib. This is not thread-safe. */
-      g_setenv ("HOME", g_get_home_dir (), TRUE);
-
       libdbus_call.conn = dbus_connection_open_private (connectable_address,
                                                         &libdbus_call.error);
       g_assert_cmpstr (libdbus_call.error.name, ==, NULL);
@@ -464,10 +441,6 @@ do_test_server_auth (InteropFlags flags)
   g_test_skip ("Testing interop with libdbus not supported");
 #endif /* !HAVE_DBUS1 */
 
-  /* No practical effect, just to avoid -Wunused-label under some
-   * combinations of #ifdefs */
-  goto out;
-
 out:
   if (server != NULL)
     g_dbus_server_stop (server);
@@ -519,12 +492,6 @@ test_server_auth_external (void)
   do_test_server_auth (INTEROP_FLAGS_EXTERNAL);
 }
 
-static void
-test_server_auth_external_require_same_user (void)
-{
-  do_test_server_auth (INTEROP_FLAGS_EXTERNAL | INTEROP_FLAGS_REQUIRE_SAME_USER);
-}
-
 static void
 test_server_auth_sha1 (void)
 {
@@ -537,55 +504,11 @@ test_server_auth_sha1_tcp (void)
   do_test_server_auth (INTEROP_FLAGS_SHA1 | INTEROP_FLAGS_TCP);
 }
 
-static void
-test_server_path_in_use (void)
-{
-#ifdef G_OS_UNIX
-#ifndef UNIX_PATH_MAX
-#define UNIX_PATH_MAX G_SIZEOF_MEMBER (struct sockaddr_un, sun_path)
-#endif
-  GError *error = NULL;
-  GString *path = NULL;
-  gchar *listenable_address = NULL;
-  GDBusServer *server = NULL;
-  gchar *guid = NULL;
-  gchar *escaped = NULL;
-
-  path = g_string_new (g_get_tmp_dir ());
-  guid = g_dbus_generate_guid ();
-  while (path->len < UNIX_PATH_MAX - 1)
-    g_string_append_c (path, 'x');
-  escaped = g_dbus_address_escape_value (path->str);
-  listenable_address = g_strdup_printf ("unix:%s=%s", "dir", escaped);
-  g_assert_cmpint (g_mkdir_with_parents (path->str, 0700), ==, 0);
-  g_string_free (path, TRUE);
-
-  server = g_dbus_server_new_sync (listenable_address,
-                                   G_DBUS_SERVER_FLAGS_RUN_IN_THREAD,
-                                   guid,
-                                   NULL,
-                                   NULL,
-                                   &error);
-  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_ADDRESS_IN_USE);
-  g_error_free (error);
-
-  if (server != NULL)
-    g_dbus_server_stop (server);
-
-  g_clear_object (&server);
-  g_free (guid);
-  g_free (escaped);
-  g_free (listenable_address);
-#else
-  g_test_skip ("unix: addresses only work on Unix");
-#endif
-}
-
 int
 main (int   argc,
       char *argv[])
 {
-  g_test_init (&argc, &argv, G_TEST_OPTION_ISOLATE_DIRS, NULL);
+  g_test_init (&argc, &argv, NULL);
 
   g_test_add_func ("/gdbus/server-auth", test_server_auth);
   g_test_add_func ("/gdbus/server-auth/abstract", test_server_auth_abstract);
@@ -593,10 +516,8 @@ main (int   argc,
   g_test_add_func ("/gdbus/server-auth/anonymous", test_server_auth_anonymous);
   g_test_add_func ("/gdbus/server-auth/anonymous/tcp", test_server_auth_anonymous_tcp);
   g_test_add_func ("/gdbus/server-auth/external", test_server_auth_external);
-  g_test_add_func ("/gdbus/server-auth/external/require-same-user", test_server_auth_external_require_same_user);
   g_test_add_func ("/gdbus/server-auth/sha1", test_server_auth_sha1);
   g_test_add_func ("/gdbus/server-auth/sha1/tcp", test_server_auth_sha1_tcp);
-  g_test_add_func ("/gdbus/server-auth/path-in-use", test_server_path_in_use);
 
   return g_test_run();
 }
