@@ -669,7 +669,7 @@ g_static_rec_mutex_get_rec_mutex_impl (GStaticRecMutex* mutex)
   if (!g_thread_supported ())
     return NULL;
 
-  result = g_atomic_pointer_get (&mutex->mutex.mutex);
+  result = (GRecMutex *) g_atomic_pointer_get (&mutex->mutex.mutex);
 
   if (!result)
     {
@@ -680,7 +680,7 @@ g_static_rec_mutex_get_rec_mutex_impl (GStaticRecMutex* mutex)
         {
           result = g_slice_new (GRecMutex);
           g_rec_mutex_init (result);
-          g_atomic_pointer_set (&mutex->mutex.mutex, result);
+          g_atomic_pointer_set (&mutex->mutex.mutex, (GMutex *) result);
         }
 
       G_UNLOCK (g_static_mutex);
