@@ -682,7 +682,7 @@ gboolean
   volatile gsize *value_location = location;
   gboolean need_init = FALSE;
   g_mutex_lock (&g_once_mutex);
-  if (g_atomic_pointer_get (value_location) == NULL)
+  if (g_atomic_pointer_get (value_location) == 0)
     {
       if (!g_slist_find (g_once_init_list, (void*) value_location))
         {
@@ -718,7 +718,7 @@ void
 {
   volatile gsize *value_location = location;
 
-  g_return_if_fail (g_atomic_pointer_get (value_location) == NULL);
+  g_return_if_fail (g_atomic_pointer_get (value_location) == 0);
   g_return_if_fail (result != 0);
 
   g_atomic_pointer_set (value_location, result);
