@@ -37,6 +37,7 @@
 #include "gslice.h"
 #include "gdatetime.h"
 #include "gdate.h"
+#include "genviron.h"
 
 #ifdef G_OS_WIN32
 
@@ -196,6 +197,8 @@ struct _GTimeZone
 
 G_LOCK_DEFINE_STATIC (time_zones);
 static GHashTable/*<string?, GTimeZone>*/ *time_zones;
+G_LOCK_DEFINE_STATIC (tz_local);
+static GTimeZone *tz_local = NULL;
 
 #define MIN_TZYEAR 1916 /* Daylight Savings started in WWI */
 #define MAX_TZYEAR 2999 /* And it's not likely ever to go away, but
@@ -1200,21 +1203,14 @@ parse_mwd_boundary (gchar **pos, TimeZoneDate *boundary)
   return TRUE;
 }
 
-/*
- * This parses two slightly different ways of specifying
- * the Julian day:
- *
- * - ignore_leap == TRUE
- *
- *   Jn   This specifies the Julian day with n between 1 and 365. Leap days
- *        are not counted. In this format, February 29 can't be represented;
- *        February 28 is day 59, and March 1 is always day 60.
- *
- * - ignore_leap == FALSE
- *
- *   n   This specifies the zero-based Julian day with n between 0 and 365.
- *       February 29 is counted in leap years.
- */
+/* Different implementations of tzset interpret the Julian day field
+   differently. For example, Linux specifies that it should be 1-based
+   (1 Jan is JD 1) for both Jn and n formats, while zOS and BSD
+   specify that a Jn JD is 1-based while an n JD is 0-based. Rather
+   than trying to follow different specs, we will follow GDate's
+   practice thatIn order to keep it simple, we will follow Linux's
+   practice. */
+
 static gboolean
 parse_julian_boundary (gchar** pos, TimeZoneDate *boundary,
                        gboolean ignore_leap)
@@ -1228,20 +1224,8 @@ parse_julian_boundary (gchar** pos, TimeZoneDate *boundary,
       day += *(*pos)++ - '0';
     }
 
-  if (ignore_leap)
-    {
-      if (day < 1 || 365 < day)
-        return FALSE;
-      if (day >= 59)
-        day++;
-    }
-  else
-    {
-      if (day < 0 || 365 < day)
-        return FALSE;
-      /* GDate wants day in range 1->366 */
-      day++;
-    }
+  if (day < 1 || 365 < day)
+    return FALSE;
 
   g_date_clear (&date, 1);
   g_date_set_julian (&date, day);
@@ -1250,6 +1234,9 @@ parse_julian_boundary (gchar** pos, TimeZoneDate *boundary,
   boundary->mday = (int) g_date_get_day (&date);
   boundary->wday = 0;
 
+  if (!ignore_leap && day >= 59)
+    boundary->mday++;
+
   return TRUE;
 }
 
@@ -1271,13 +1258,13 @@ parse_tz_boundary (const gchar  *identifier,
   else if (*pos == 'J')
     {
       ++pos;
-      if (!parse_julian_boundary (&pos, boundary, TRUE))
+      if (!parse_julian_boundary (&pos, boundary, FALSE))
         return FALSE ;
     }
   /* Julian date which counts Feb 29 in leap years */
   else if (*pos >= '0' && '9' >= *pos)
     {
-      if (!parse_julian_boundary (&pos, boundary, FALSE))
+      if (!parse_julian_boundary (&pos, boundary, TRUE))
         return FALSE;
     }
   else
@@ -1501,7 +1488,8 @@ rules_from_identifier (const gchar   *identifier,
  * the local time.
  *
  * In UNIX, the `TZ` environment variable typically corresponds
- * to the name of a file in the zoneinfo database, or string in
+ * to the name of a file in the zoneinfo database, an absolute path to a file
+ * somewhere else, or a string in
  * "std offset [dst [offset],start[/time],end[/time]]" (POSIX) format.
  * There  are  no spaces in the specification. The name of standard
  * and daylight savings time zone must be three or more alphabetic
@@ -1672,10 +1660,20 @@ g_time_zone_new (const gchar *identifier)
  *
  * Since: 2.26
  **/
+static gpointer
+g_time_zone_utc_init (gpointer data)
+{
+  return g_time_zone_new ("UTC");
+}
+
 GTimeZone *
 g_time_zone_new_utc (void)
 {
-  return g_time_zone_new ("UTC");
+  static GOnce utc_once = G_ONCE_INIT;
+
+  g_once (&utc_once, g_time_zone_utc_init, NULL);
+
+  return g_time_zone_ref ((GTimeZone *)utc_once.retval);
 }
 
 /**
@@ -1698,7 +1696,23 @@ g_time_zone_new_utc (void)
 GTimeZone *
 g_time_zone_new_local (void)
 {
-  return g_time_zone_new (getenv ("TZ"));
+  const gchar *tzenv = g_getenv ("TZ");
+  GTimeZone *tz;
+
+  G_LOCK (tz_local);
+
+  /* Is time zone changed and must be flushed? */
+  if (tz_local && g_strcmp0 (g_time_zone_get_identifier (tz_local), tzenv))
+    g_clear_pointer (&tz_local, g_time_zone_unref);
+
+  if (tz_local == NULL)
+    tz_local = g_time_zone_new (tzenv);
+
+  tz = g_time_zone_ref (tz_local);
+
+  G_UNLOCK (tz_local);
+
+  return tz;
 }
 
 /**
