@@ -139,12 +139,11 @@ const gchar * const g_utf8_skip = utf8_skip_data;
  * Returns: (transfer none) (nullable): a pointer to the found character or %NULL.
  */
 gchar *
-g_utf8_find_prev_char (const gchar *str,
-		       const gchar *p)
+g_utf8_find_prev_char (const char *str,
+		       const char *p)
 {
-  while (p > str)
+  for (--p; p >= str; --p)
     {
-      --p;
       if ((*p & 0xc0) != 0x80)
 	return (gchar *)p;
     }
@@ -686,6 +685,11 @@ g_utf8_get_char_validated (const gchar *p,
 
   result = g_utf8_get_char_extended (p, max_len);
 
+  /* Disallow codepoint U+0000 as it’s a nul byte,
+   * and all string handling in GLib is nul-terminated */
+  if (result == 0 && max_len > 0)
+    return (gunichar) -2;
+
   if (result & 0x80000000)
     return result;
   else if (!UNICODE_VALID (result))
