@@ -818,7 +818,6 @@ test_GDateTime_new_from_iso8601_2 (void)
     { TRUE, "1970-01-01T00:00:17.1234Z", 1970, 1, 1, 0, 0, 17, 123400, 0 },
     { TRUE, "1970-01-01T00:00:17.123456Z", 1970, 1, 1, 0, 0, 17, 123456, 0 },
     { TRUE, "1980-02-22T12:36:00+02:00", 1980, 2, 22, 12, 36, 0, 0, 2 * G_TIME_SPAN_HOUR },
-    { TRUE, "1990-12-31T15:59:60-08:00", 1990, 12, 31, 15, 59, 59, 0, -8 * G_TIME_SPAN_HOUR },
     { FALSE, "   ", 0, 0, 0, 0, 0, 0, 0, 0 },
     { FALSE, "x", 0, 0, 0, 0, 0, 0, 0, 0 },
     { FALSE, "123x", 0, 0, 0, 0, 0, 0, 0, 0 },
@@ -2432,8 +2431,8 @@ test_posix_parse (void)
   g_date_time_unref (gdt2);
   g_time_zone_unref (tz);
 
-  tz = g_time_zone_new ("NZST-12:00:00NZDT-13:00:00,279,76");
-  g_assert_cmpstr (g_time_zone_get_identifier (tz), ==, "NZST-12:00:00NZDT-13:00:00,279,76");
+  tz = g_time_zone_new ("NZST-12:00:00NZDT-13:00:00,280,77");
+  g_assert_cmpstr (g_time_zone_get_identifier (tz), ==, "NZST-12:00:00NZDT-13:00:00,280,77");
   g_assert_cmpstr (g_time_zone_get_abbreviation (tz, 0), ==, "NZST");
   g_assert_cmpint (g_time_zone_get_offset (tz, 0), ==, 12 * 3600);
   g_assert (!g_time_zone_is_dst (tz, 0));
@@ -2533,48 +2532,6 @@ test_posix_parse (void)
   g_date_time_unref (gdt1);
   g_date_time_unref (gdt2);
   g_time_zone_unref (tz);
-
-  tz = g_time_zone_new ("VIR-00:30");
-  g_assert_cmpstr (g_time_zone_get_identifier (tz), ==, "VIR-00:30");
-  g_assert_cmpstr (g_time_zone_get_abbreviation (tz, 0), ==, "VIR");
-  g_assert_cmpint (g_time_zone_get_offset (tz, 0), ==, (30 * 60));
-  g_assert_false (g_time_zone_is_dst (tz, 0));
-
-  tz = g_time_zone_new ("VIR-00:30VID,0/00:00:00,365/23:59:59");
-  g_assert_cmpstr (g_time_zone_get_identifier (tz), ==, "VIR-00:30VID,0/00:00:00,365/23:59:59");
-  g_assert_cmpstr (g_time_zone_get_abbreviation (tz, 0), ==, "VIR");
-  g_assert_cmpint (g_time_zone_get_offset (tz, 0), ==, (30 * 60));
-  g_assert_false (g_time_zone_is_dst (tz, 0));
-  g_assert_cmpstr (g_time_zone_get_abbreviation (tz, 1), ==, "VID");
-  g_assert_cmpint (g_time_zone_get_offset (tz, 1), ==, (90 * 60));
-  g_assert_true (g_time_zone_is_dst (tz, 1));
-
-  tz = g_time_zone_new ("VIR-02:30VID,0/00:00:00,365/23:59:59");
-  g_assert_cmpstr (g_time_zone_get_identifier (tz), ==, "VIR-02:30VID,0/00:00:00,365/23:59:59");
-  g_assert_cmpstr (g_time_zone_get_abbreviation (tz, 0), ==, "VIR");
-  g_assert_cmpint (g_time_zone_get_offset (tz, 0), ==, (150 * 60));
-  g_assert_false (g_time_zone_is_dst (tz, 0));
-  g_assert_cmpstr (g_time_zone_get_abbreviation (tz, 1), ==, "VID");
-  g_assert_cmpint (g_time_zone_get_offset (tz, 1), ==, (210 * 60));
-  g_assert_true (g_time_zone_is_dst (tz, 1));
-
-  tz = g_time_zone_new ("VIR-02:30VID-04:30,0/00:00:00,365/23:59:59");
-  g_assert_cmpstr (g_time_zone_get_identifier (tz), ==, "VIR-02:30VID-04:30,0/00:00:00,365/23:59:59");
-  g_assert_cmpstr (g_time_zone_get_abbreviation (tz, 0), ==, "VIR");
-  g_assert_cmpint (g_time_zone_get_offset (tz, 0), ==, (150 * 60));
-  g_assert_false (g_time_zone_is_dst (tz, 0));
-  g_assert_cmpstr (g_time_zone_get_abbreviation (tz, 1), ==, "VID");
-  g_assert_cmpint (g_time_zone_get_offset (tz, 1), ==, (270 * 60));
-  g_assert_true (g_time_zone_is_dst (tz, 1));
-
-  tz = g_time_zone_new ("VIR-12:00VID-13:00,0/00:00:00,365/23:59:59");
-  g_assert_cmpstr (g_time_zone_get_identifier (tz), ==, "VIR-12:00VID-13:00,0/00:00:00,365/23:59:59");
-  g_assert_cmpstr (g_time_zone_get_abbreviation (tz, 0), ==, "VIR");
-  g_assert_cmpint (g_time_zone_get_offset (tz, 0), ==, (720 * 60));
-  g_assert_false (g_time_zone_is_dst (tz, 0));
-  g_assert_cmpstr (g_time_zone_get_abbreviation (tz, 1), ==, "VID");
-  g_assert_cmpint (g_time_zone_get_offset (tz, 1), ==, (780 * 60));
-  g_assert_true (g_time_zone_is_dst (tz, 1));
 }
 
 static void
@@ -2628,11 +2585,11 @@ test_identifier (void)
   g_time_zone_unref (tz);
 
   /* System timezone. We can’t change this, but we can at least assert that
-   * the identifier is non-NULL and doesn’t start with a slash. */
+   * the identifier is non-NULL and non-empty. */
   tz = g_time_zone_new (NULL);
+  g_test_message ("System time zone identifier: %s", g_time_zone_get_identifier (tz));
   g_assert_nonnull (g_time_zone_get_identifier (tz));
   g_assert_cmpstr (g_time_zone_get_identifier (tz), !=, "");
-  g_assert_true (*g_time_zone_get_identifier (tz) != '/');
   g_time_zone_unref (tz);
 
   /* Local timezone tests. */
